/***************************************************************************
 *   Copyright (C) 2018 by Stefan Kebekus                             *
 *   stefan.kebekus@math.uni-freiburg.de                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef ECGRAPHICSSCENE
#define ECGRAPHICSSCENE

#include <QGraphicsScene>

class ec_realFreePoint;
class ec_realPoint;

/**
 * Re-implementation QGraphicsScene which improves user experience on devices
 * with touch screens
 *
 * The elliptic curve plotter allows the user to drag points to, from and along
 * a given elliptic curve. While this works great for people using a mouse,
 * users with touch screens (e.g. Android tablet computers) find it hard to hit
 * the point with the fingers ---the points are just too small.
 *
 * This drop-in replacement for QGraphicsScene reimplementats the
 * mouse event handlers, in order to allow more 'fuzz'. Users can move
 * points by clicking next to a point rather than on a point to move it.
 *
 * @author Stefan Kebekus
 */


class ecGraphicsScene : public QGraphicsScene
{
  Q_OBJECT
    
 public:
  /**
   * Standard constructor, reimplemented from QGraphicsScene
   */
  ecGraphicsScene(QObject *parent = nullptr);

  /**
   * Reimplemented from QGraphicsScene
   */
  void mousePressEvent(QGraphicsSceneMouseEvent * mouseEvent) override;

  /**
   * Reimplemented from QGraphicsScene
   */
  void mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent) override;

  /**
   * Reimplemented from QGraphicsScene
   */
  void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent) override;

 private:
  ec_realFreePoint *grabber;
};

#endif
